//% color="#ff9f06" iconWidth=50 iconHeight=40
namespace RFID_Module{
    
    //% block="Initialize RFID module with I2C address 0x7C" blockType="command"  
    export function RFID_init(parameter: any, block: any) { 
        Generator.addImport('from pinpong.board import Board,Pin');
        Generator.addImport('from pinpong.libs.dfrobot_pn7150 import *');
        Generator.addInit(`board_begin`,`Board().begin()`);
        Generator.addInit(`RFID_object`,`PN7150 = DFRobot_PN7150(i2c_addr=PN7150_I2C_ADDR, bus_num=4)`);
        Generator.addInit(`RFID_connect`,`
while (PN7150.connect() == False):
	print("init fail!")
	time.sleep(1)
print("successful connection.")
`);
        Generator.addInit(`RFID_mode`,`PN7150.mode_rw()`);
    }



    //% block="Write [DATA] to byte [BYTE_NUM] of RFID block [BLOCK_NUM]" blockType="command" 
    //% DATA.shadow="range" DATA.params.min=0 DATA.params.max=255 DATA.defl=200
    //% BYTE_NUM.shadow="dropdown" BYTE_NUM.options="BYTE_NUM"
    //% BLOCK_NUM.shadow="dropdown" BLOCK_NUM.options="BLOCK_NUM"
    export function RFID_write(parameter: any, block: any) { 
        let data=parameter.DATA.code;
        let block_num=parameter.BLOCK_NUM.code;
        let byte_num=parameter.BYTE_NUM.code;

        Generator.addInit(`RFID_write`,`
def RFID_write(block,types,data):
    protocol = PN7150.read_protocol()
    if protocol == "T2T":
        if types > 4:
            print("Byte index error")
        elif block < 4:
            print("Data block read-only")
        else:
            PN7150.t2t_write_index_data(block, types, data)
    elif protocol == "mifare":
        PN7150.write_index_data(block, types, data)
`);
        Generator.addCode(`RFID_write(${block_num}, ${byte_num}, ${data})`);
    }



    //% block="Read all data from block [BLOCK_NUM] of the RFID module(string)" blockType="reporter" 
    //% BLOCK_NUM.shadow="dropdown" BLOCK_NUM.options="BLOCK_NUM"
    export function RFID_read_all(parameter: any, block: any) { 
        let block_num=parameter.BLOCK_NUM.code;

        Generator.addInit(`RFID_read_all`,`
def RFID_read_all(block):
    protocol = PN7150.read_protocol()
    if protocol == "T2T":
        return PN7150.t2t_read(block)
    elif protocol == "mifare":
        return PN7150.read_data(block)
`);
        Generator.addCode(`RFID_read_all(${block_num})`);
    }



    //% block="Read byte [BYTE_NUM] of RFID block [BLOCK_NUM]" blockType="reporter" 
    //% BLOCK_NUM.shadow="dropdown" BLOCK_NUM.options="BLOCK_NUM"
    //% BYTE_NUM.shadow="dropdown" BYTE_NUM.options="BYTE_NUM"
    export function RFID_read(parameter: any, block: any) { 
        let block_num=parameter.BLOCK_NUM.code;
        let byte_num=parameter.BYTE_NUM.code;

        Generator.addInit(`RFID_read`,`
def RFID_read(block,types):
    protocol = PN7150.read_protocol()
    if protocol == "T2T":
        return PN7150.t2t_read(block, types)
    elif protocol == "mifare":
        return PN7150.read_data(block, types)
`);
        Generator.addCode(`RFID_read(${block_num}, ${byte_num})`);
    }



    //% block="Read the UID of the tag card" blockType="reporter" 
    export function RFID_read_UID(parameter: any, block: any) { 

        Generator.addCode(`PN7150.read_uid()`);
    }


    //% block="Detected card. UID:[UID]?" blockType="boolean" 
    //% UID.shadow="string" UID.defl="acee50e3"
    export function RFID_SCAN_UID(parameter: any, block: any) { 
        
        let uid=parameter.UID.code;
        Generator.addCode(`PN7150.scan(${uid})`);
    }


    //% block="Detected card?" blockType="boolean" 
    export function RFID_SCAN(parameter: any, block: any) { 

        Generator.addCode(`PN7150.scan()`);
    }

}

